-- Filename:     counter_4bit.vhd
-- Filetype:     VHDL Design File
-- Date:         9 mar 2014
-- Update:       -
-- Description:  A behavioral description of a 4 bit binary counter
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.2
-- Copyright:    (c)2014, De Haagse Hogeschool

-- The design file contains the description of a 4-bit binary
-- up counter with an enable and a synchronous clear. The 
-- counter provides a terminal count signal for signalling that
-- the counter has reached its maximum value.

-- Use library std_logic_1164
-- Use library numeric_std for addition
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- The entity
entity counter_4bit is
    port (clk    : in std_logic;
			 areset : in std_logic;
          en     : in std_logic;
          clear  : in std_logic;
          q      : out unsigned(3 downto 0);
          tc     : out std_logic
         );
end entity counter_4bit;

-- The architecture
architecture rtl of counter_4bit is
-- Internal count value since port q is of out-mode
-- and you cannot read from an out-mode port.
signal count : unsigned(3 downto 0);
begin

    process (clk, areset) is
    begin
        if areset = '1' then
            count <= "0000";
        -- On positive edge
        elsif rising_edge(clk) then
            -- If clear is activated, clear the counter
            -- If en is activated, increment the counter
            if clear = '1' then
                count <= "0000";
            elsif en = '1' then
                count <= count + 1;
            end if;
        end if;
    end process;

    -- Output the value of the counter
    q <= count;
	 --q(3) <= count(3) after 2 ns;
	 --q(2) <= count(2) after 5 ns;
	 --q(1) <= count(1) after 3 ns;
	 --q(0) <= count(0) after 1 ns;

    -- Signal if the counter has reached the maximum count
    --cout <= '1' when count = "1111" and en = '1' else '0';
    tc <= '1' after 5 ns when count = "1111" and en = '1' else '0' after 3 ns;
	
end rtl;
