-- Filename:     tb_counter_4bit.vhd
-- Filetype:     VHDL Test Bench File
-- Date:         9 mar 2014
-- Update:       -
-- Description:  A test bench for a 4 bit binary counter
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.2
-- Copyright:    (c)2014, De Haagse Hogeschool

-- The design file contains the description of a test bench
-- for a 4-bit binary up counter with an enable and a 
-- synchronous clear. The counter provides a terminal count signal
-- for signalling that the counter has reached its maximum value.

-- Use library std_logic_1164
-- Use library numeric_std for addition
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_counter_4bit is
end entity tb_counter_4bit;

architecture sim of tb_counter_4bit is
-- The signals to be monitored
signal clk    : std_logic;
signal areset : std_logic;
signal en     : std_logic;
signal clear  : std_logic;
signal q      : unsigned(3 downto 0);
signal tc     : std_logic;

-- Component declaration of the counter
component counter_4bit is
    port (clk    : in std_logic;
          areset : in std_logic;
          en     : in std_logic;
          clear  : in std_logic;
          q      : out unsigned(3 downto 0);
          tc     : out std_logic
         );
end component counter_4bit;

begin

	dut: counter_4bit
	port map (clk => clk, areset => areset, en => en, clear => clear, q => q, tc => tc);
	
	clockgen: process is
	begin
		clk <= '0';
		wait for 10 ns;
		clk <= '1';
		wait for 10 ns;
	end process clockgen;

	datagen: process is
	begin
		-- Disable and do not reset, wait a while...
		clear <= '0';
		en <= '0';
		areset <= '0';
		wait until clk = '1';
		wait until clk = '1';
		wait for 1 ns;

		-- Force reset
		areset <= '1';
		wait until clk = '1';
		wait for 1 ns;

		-- Unforce reset, enable counting and
		-- wait until we've reached the maximum
		-- count value...
		en <= '1';
		areset <= '0';
		wait until q = "1111";
		wait until clk = '0';
		wait for 1 ns;

		-- Disable counting...
		en <= '0';
		wait until clk = '1';
		wait until clk = '1';
		wait for 1 ns;
		
		-- And enable again...
		en <= '1';
		wait;
		
	end process datagen;
	
end sim;
